#' Compute Nested Case-control Sampling Probabilities
#'
#' @param etime survival time to event where event is either case or censored
#' @param outcome binary outcome status (1 = case; 0 = non-case)
#' @param m number of controls per case
#' @param matvar a vector or matrix of matching variable(s); If NULL, probabilities are only based on matching by \code{etime}; Default is NULL.
#' @keywords ahazcal
#' @return a vector of probabilities of being included in nested case-control samples
#' @export
#' @seealso 
#' @examples sample_data$incl.prob = with(sample_data, ncc.sample.prob(etime = eventime, outcome = ind.fail, m = 1))
ncc.sample.prob = function(etime, outcome, m, matvar = NULL){
  N = length(outcome) # full cohort size
  
  if(is.null(matvar)){
    nrisk = mapply(function(i){ sum(etime >= etime[i], na.rm = T) }, 1:N)
    incl_prob = mapply(function(i){1-prod(1-m/(nrisk[which(nrisk > m+1
                                                           & (etime <= etime[i]) 
                                                           & (outcome == 1))]-1))}, 1:N)
  }
  if(!is.null(matvar)){
    if(!is.matrix(matvar)) matvar = matrix(matvar, ncol = 1)
    if(is.matrix(matvar)) matvar = matrix(apply(matvar, 1, paste, collapse = ""), ncol = 1)
    nrisk = mapply(function(i){ sum(etime >= etime[i] 
                                    & matvar == matvar[i,], na.rm = T) }, 1:N)
    incl_prob = mapply(function(i){1-prod(1-m/(nrisk[which(nrisk > m+1
                                                           & (etime <= etime[i]) 
                                                           & (outcome == 1)
                                                           & matvar == matvar[i,])]-1))}, 1:N)
  }

  incl_prob[outcome == 1] = 1
  
  return(cbind(nrisk = nrisk, incl_prob = incl_prob))
}

