

addHazard <- function(data, ncc.subset, outcome.var, time.vars, timeDep.covars, timeIndep.covars,
                      anc.covars, risk.time, risk.time0=0, nCntlPerCase=1,
                      inclProb.var=NULL, nrisk.var=NULL, 
                      print=1, min.inclProb=1e-6, control=NULL) {

  check_data(data)
  check_subset(ncc.subset, "ncc.subset", nrow(data)) 
  check_outcome(outcome.var, data)
  check_time_vars(data, time.vars)
  tmp <- colnames(data)
  check_vars(timeDep.covars, "timeDep.covars", tmp, minlen=1)
  check_vars(timeIndep.covars, "timeIndep.covars", tmp, minlen=1)
  check_vars(anc.covars, "anc.covars", tmp)
  check_inclProb(inclProb.var, data)
  check_numeric(risk.time, "risk.time")
  check_numeric(risk.time0, "risk.time0", pos=0)
  check_numeric(nCntlPerCase, "nCntlPerCase")
  control <- check_control(control) 

  obj.list <- list(ncc.subset=ncc.subset, outcome.var=outcome.var, time.vars=time.vars, timeDep.covars=timeDep.covars, 
              timeIndep.covars=timeIndep.covars, anc.covars=anc.covars, tau=risk.time, inclProb.var=inclProb.var,
              nCntlPerCase=nCntlPerCase, tau0=risk.time0, min.per.miss=0.9, min.nrow=5, print=print,
              min.inclProb=min.inclProb, nrisk.var=nrisk.var, aux.stat.vars=NULL,
              control=control)

  ret <- addhaz_main(data, obj.list) 
  ret
}


addhaz_main <- function(data, obj.list) {
 
  tmp              <- add_setupData(data, obj.list)
  data             <- tmp$data
  obj.list         <- tmp$obj.list
  tmp              <- NULL

  # Check the model columns, remove variables that need to be removed
  obj.list <- add_checkCols(data, obj.list)

  ncc.subset       <- obj.list$ncc.subset
  timeDep.covars   <- obj.list[["timeDep.covars", exact=TRUE]]
  timeIndep.covars <- obj.list[["timeIndep.covars", exact=TRUE]]
  anc.covars       <- obj.list[["anc.covars", exact=TRUE]]
  incl.prob        <- data[, obj.list$inclProb.var, drop=TRUE]
  wgts             <- 1/incl.prob

  allcovars        <- c(timeDep.covars, timeIndep.covars, anc.covars)
  
  # Impute missing
  impute.dep      <- getVarsToImpute(data, ncc.subset, timeDep.covars, min.per.miss=obj.list$min.per.miss) 
  impute.indep    <- getVarsToImpute(data, ncc.subset, timeIndep.covars, min.per.miss=obj.list$min.per.miss) 
  impute.vars     <- c(impute.dep, impute.indep)
  if (!length(impute.vars)) stop("ERROR: no covariates to impute")

  data2 <- data
  covarsForImpute <- allcovars[!(allcovars %in% impute.vars)]
  if (length(impute.dep))   data2 <- predictMissingVars(data2, ncc.subset, wgts, impute.dep,   covarsForImpute) 
  if (length(impute.indep)) data2 <- predictMissingVars(data2, ncc.subset, wgts, impute.indep, covarsForImpute) 

  # Obtain Auxiliary Statistics (Influence Functions)
  aux <- add_aux(data2, obj.list)

  # Calibrate weights
  cal <- add_calibrate(data2, aux, obj.list)

  # Fit additive hazards model
  ret <- add_hazModel(data, aux, cal, obj.list) 
  ret <- addhaz_setRet(ret, obj.list)

  class(ret) <- "addHazard"  

  ret

}

addhaz_setNames <- function(x) {

  nms      <- names(x)
  nms      <- gsub("Beta:", "", nms, fixed=TRUE)
  nms      <- gsub("gamma:", "", nms, fixed=TRUE)
  tmp      <- nms %in% "0"
  nms[tmp] <- "(Intercept)"
  names(x) <- nms
  x
}

addhaz_setRet <- function(obj, obj.list) {

  timeDep.var.info.list   <- obj.list[["timeDep.var.info.list", exact=TRUE]]
  timeIndep.var.info.list <- obj.list[["timeIndep.var.info.list", exact=TRUE]]

  x                         <- obj$fit.with.cal
  x$est                     <- addhaz_setNames(x$est)
  x$se                      <- addhaz_setNames(x$se)
  x$timeDep.var.info.list   <- timeDep.var.info.list
  x$timeIndep.var.info.list <- timeIndep.var.info.list

  y                         <- obj$fit.no.cal
  y$est                     <- addhaz_setNames(y$est)
  y$se                      <- addhaz_setNames(y$se)
  y$timeDep.var.info.list   <- timeDep.var.info.list
  y$timeIndep.var.info.list <- timeIndep.var.info.list

  ret <- list(with.calibration=x, without.calibration=y)

  ret
}

add_hazModel <- function(data, aux, cal, obj.list) {

  outcome   <- data[, obj.list$outcome.var, drop=TRUE]
  tvars     <- obj.list$time.vars
  ntvars    <- length(tvars)
  time1     <- data[, tvars[1], drop=TRUE]
  time2     <- NULL
  if (ntvars > 1) time2 <- data[, tvars[2], drop=TRUE]
  tau       <- obj.list$tau
  tau0      <- obj.list$tau0
  ind.ph2   <- obj.list$ncc.subset
  m         <- obj.list$nCntlPerCase
  incl.prob <- data[, obj.list$inclProb.var, drop=TRUE]
  nrisk     <- data[, obj.list$nrisk.var, drop=TRUE]
  xvars     <- obj.list[["timeDep.covars", exact=TRUE]]
  zvars     <- obj.list[["timeIndep.covars", exact=TRUE]]
  X         <- as.matrix(data[, xvars, drop=FALSE])
  Z         <- as.matrix(data[, zvars, drop=FALSE])

  # No calibration
  fit0 <-  ahazw(outcome, time1, time2=time2, tau, tau0=tau0, ind.ph2, m, X, Z, 
                 incl.prob=incl.prob, nrisk=nrisk, A=NULL, eta=NULL, varest=TRUE) 

  # With calibration
  fit1 <-  ahazw(outcome, time1, time2=time2, tau, tau0=tau0, ind.ph2, m, X, Z, 
                 incl.prob=incl.prob, nrisk=nrisk, A=aux, eta=attr(cal, "eta"), varest=TRUE) 

  list(fit.no.cal=fit0, fit.with.cal=fit1)

}


add_calibrate <- function(data, aux, obj.list) {

  subset  <- obj.list$ncc.subset
  wgts    <- 1/data[subset, obj.list$inclProb.var, drop=TRUE]
  control <- obj.list$control

  ret <- rakecal(aux[subset, , drop=FALSE], wgts, colSums(aux), eta=TRUE, 
                 max_iter=control$maxit, EPS=control$eps, conv.type=control$conv)

  ret
}

add_aux <- function(data, obj.list) {
 

  yvar    <- obj.list$outcome.var
  outcome <- data[, yvar, drop=TRUE]
  tvars   <- obj.list$time.vars
  ntvars  <- length(tvars)
  time1   <- data[, tvars[1], drop=TRUE]
  time2   <- NULL
  if (ntvars > 1) time2 <- data[, tvars[2], drop=TRUE]
  tau     <- obj.list$tau
  tau0    <- obj.list$tau0
  xvars   <- obj.list[["timeDep.covars", exact=TRUE]]
  zvars   <- obj.list[["timeIndep.covars", exact=TRUE]]
  X       <- NULL
  Z       <- NULL
  if (length(xvars)) X <- as.matrix(data[, xvars, drop=FALSE])
  if (length(zvars)) Z <- as.matrix(data[, zvars, drop=FALSE])
  if (!length(X)) stop("ERROR X=NULL")
  if (!length(Z)) stop("ERROR Z=NULL")

  aux   <- aux.ahaz(outcome, time1, time2=time2, tau, tau0=tau0, X, Z)
  avars <- add_getAuxVars(obj.list)
  if (!length(avars)) {
    # Use all variables
    ret <- cbind(1, aux)
  } else {
    # Check that these variables exist
    tmp <- !(avars %in% colnames(aux))
    if (any(tmp)) {
      print(avars[tmp])
      stop("ERROR aux.vars")
    }
    ret <- cbind(1, aux[, avars, drop=FALSE])
  }
  ret
}

add_getAuxVars <- function(obj.list) {

  avars   <- obj.list[["aux.stat.vars", exact=TRUE]]
  if (is.null(avars)) return(NULL)

  xvars   <- obj.list[["timeDep.covars", exact=TRUE]]
  zvars   <- obj.list[["timeIndep.covars", exact=TRUE]]
  xlist   <- obj.list[["timeDep.var.info.list", exact=TRUE]]
  zlist   <- obj.list[["timeIndep.var.info.list", exact=TRUE]]
  ret     <- NULL
  for (v in avars) {
    if (v %in% xvars) {
      tmp <- xlist[[v, exact=TRUE]]
      if (is.null(tmp)) stop("ERROR 1")
      dvars <- tmp[["dvars", exact=TRUE]]
      if (is.null(dvars)) stop("ERROR 2")
      vv    <- paste0("Beta:", dvars)
    } else if (v %in% zvars) {
      tmp <- zlist[[v, exact=TRUE]]
      if (is.null(tmp)) stop("ERROR 3")
      dvars <- tmp[["dvars", exact=TRUE]]
      if (is.null(dvars)) stop("ERROR 4")
      vv    <- paste0("gamma:", dvars)
    } else {
      stop("ERROR 99")
    }
    ret <- c(ret, vv)
  }
  ret
  
}

add_setupData <- function(data, obj.list) {

  nr0              <- nrow(data)
  ncc.subset       <- obj.list$ncc.subset
  outcome          <- obj.list$outcome.var
  time.vars        <- obj.list$time.vars
  timeDep.covars   <- obj.list[["timeDep.covars", exact=TRUE]]
  timeIndep.covars <- obj.list[["timeIndep.covars", exact=TRUE]]
  anc.covars       <- obj.list[["anc.covars", exact=TRUE]]
  inclProb.var     <- obj.list[["inclProb.var", exact=TRUE]]
  nrisk.var        <- obj.list[["nrisk.var", exact=TRUE]]
  min.nrow         <- obj.list$min.nrow
  min.per.miss     <- obj.list$min.per.miss
  print            <- obj.list$print
  min.inclProb     <- obj.list$min.inclProb
  DIAGFLAG         <- obj.list[["DIAGNOSTIC", exact=TRUE]]  
  if (!length(DIAGFLAG)) DIAGFLAG <- 0

  timeDep.covars   <- unique(timeDep.covars)
  timeIndep.covars <- unique(timeIndep.covars)
  anc.covars       <- unique(anc.covars)

  timeDep.var.info.list   <- NULL
  timeIndep.var.info.list <- NULL
  anc.var.info.list       <- NULL

  # Check for overlapping variables
  check_overlap(outcome,        time.vars,        "outcome",        "time.vars")
  check_overlap(outcome,        timeDep.covars,   "outcome",        "timeDep.covars")
  check_overlap(outcome,        timeIndep.covars, "outcome",        "timeIndep.covars")
  check_overlap(outcome,        anc.covars,       "outcome",        "anc.covars")
  check_overlap(anc.covars,     timeDep.covars,   "anc.covars",     "timeDep.covars")
  check_overlap(anc.covars,     timeIndep.covars, "anc.covars",     "timeIndep.covars")
  check_overlap(timeDep.covars, timeIndep.covars, "timeDep.covars", "timeIndep.covars")

  # Remove rows with missing values
  vars1      <- c(outcome, time.vars, inclProb.var, nrisk.var, anc.covars)
  vars2      <- c(timeDep.covars, timeIndep.covars)
  tmp        <- removeMiss(data, ncc.subset, vars1, vars2, min.nrow=min.nrow, print=print) 
  data       <- tmp$data
  ncc.subset <- tmp$ncc.subset
  tmp        <- NULL

  # Dummy vars for categorical variables are needed
  if (!DIAGFLAG) {
    if (!is.null(anc.covars)) {
      tmp <- checkAndAddDummy(data, anc.covars, print=print)
      if (length(tmp)) {
        data              <- tmp$data
        anc.covars        <- tmp$vars
        anc.var.info.list <- tmp$var.info.list
      } else {
        stop("ERROR: anc.covars are constant and have been removed")
      }
    } else {
      if (print) message("Since anc.covars were not specified, random noise will be used instead.")

      # Add random noise
      anc.covars         <- getRandomVariable("ANC", len=12)
      data[, anc.covars] <- runif(nrow(data))
    }
  }
 
  tmp <- checkAndAddDummy(data, timeDep.covars, print=print)
  if (length(tmp)) {
    data                  <- tmp$data
    timeDep.covars        <- tmp$vars
    timeDep.var.info.list <- tmp$var.info.list
  }
  tmp <- checkAndAddDummy(data, timeIndep.covars, print=print)
  if (length(tmp)) {
    data                    <- tmp$data
    timeIndep.covars        <- tmp$vars
    timeIndep.var.info.list <- tmp$var.info.list
  }

  # Compute probability of inclusion of number at risk if needed
  cond <- (!DIAGFLAG && (is.null(inclProb.var) || is.null(nrisk.var))) ||
          (DIAGFLAG && is.null(inclProb.var))
  if (cond) {
    etime <- getEventTime(data, time.vars)
    tmp   <- ncc.sample.prob(etime, data[, outcome, drop=TRUE], obj.list$nCntlPerCase)
    if (nrow(tmp) != nrow(data)) stop("ERROR calling ncc.sample.prob")
    if (is.null(inclProb.var)) {
      inclProb.var         <- getRandomVariable("INCLPROB", len=12)
      data[, inclProb.var] <- tmp[, 2]
    }
    if (is.null(nrisk.var) && !DIAGFLAG) {
      nrisk.var         <- getRandomVariable("NRISK", len=12)
      data[, nrisk.var] <- tmp[, 1]
    }
  }
  tmp <- data[, inclProb.var, drop=TRUE] < min.inclProb
  tmp[is.na(tmp)] <- FALSE
  if (any(tmp)) {
    if (print) message("Some inclusion probabilities were less than min.inclProb")
    data[tmp, inclProb.var] <- min.inclProb
  }

  # Only keep variables we need 
  vars <- c(outcome, time.vars, timeDep.covars, timeIndep.covars, anc.covars, inclProb.var, nrisk.var)
  data <- data[, vars, drop=FALSE]

  obj.list$timeDep.covars          <- timeDep.covars
  obj.list$timeIndep.covars        <- timeIndep.covars
  obj.list$anc.covars              <- anc.covars
  obj.list$inclProb.var            <- inclProb.var
  obj.list$nrisk.var               <- nrisk.var
  obj.list$timeDep.var.info.list   <- timeDep.var.info.list
  obj.list$timeIndep.var.info.list <- timeIndep.var.info.list
  obj.list$anc.var.info.list       <- anc.var.info.list
  obj.list$ncc.subset              <- ncc.subset

  list(data=data, obj.list=obj.list)

}

add_checkCols <- function(data, obj.list) {

  timeDep.covars   <- obj.list[["timeDep.covars", exact=TRUE]]
  timeIndep.covars <- obj.list[["timeIndep.covars", exact=TRUE]]
  vars             <- c(timeDep.covars, timeIndep.covars)
  
  rem <- getColsToRemove(data[, vars, drop=FALSE], add.int=1)
  if (!length(rem)) return(obj.list)

  timeDep.var.info.list   <- obj.list$timeDep.var.info.list
  timeIndep.var.info.list <- obj.list$timeIndep.var.info.list
  timeDep.var.info.list   <- updateVarInfoList(timeDep.var.info.list, rem)
  timeIndep.var.info.list <- updateVarInfoList(timeIndep.var.info.list, rem)

  tmp              <- !(timeDep.covars %in% rem)
  timeDep.covars   <- timeDep.covars[tmp]
  tmp              <- !(timeIndep.covars %in% rem)
  timeIndep.covars <- timeIndep.covars[tmp]
  if (!length(timeDep.covars)) stop("ERROR: all timeDep.covars have been removed")
  if (!length(timeIndep.covars)) stop("ERROR: all timeIndep.covars have been removed")

  obj.list$timeDep.covars          <- timeDep.covars
  obj.list$timeIndep.covars        <- timeIndep.covars
  obj.list$timeDep.var.info.list   <- timeDep.var.info.list
  obj.list$timeIndep.var.info.list <- timeIndep.var.info.list

  obj.list
}

print.addHazard <- function(x, ...) {

  lst  <- x$with.calibration
  nms  <- names(lst$est)
  mat1 <- data.frame(Variable=nms, Estimate=lst$est, SE=lst$se, stringsAsFactors=FALSE)
  rownames(mat1) <- NULL
  lst  <- x$without.calibration
  nms  <- names(lst$est)
  mat0 <- data.frame(Variable=nms, Estimate=lst$est, SE=lst$se, stringsAsFactors=FALSE)
  rownames(mat0) <- NULL

  cat("With Calibration:\n")
  print(mat1)
  cat("\n")
  cat("Without Calibration:\n")
  print(mat0)

  NULL
}