

coxHazard <- function(data, ncc.subset, outcome.var, time.vars, covars,
                      anc.covars, risk.time, risk.time0=0, nCntlPerCase=1,
                      inclProb.var=NULL, 
                      print=1, min.inclProb=1e-6, control=NULL) {

  check_data(data)
  check_subset(ncc.subset, "ncc.subset", nrow(data)) 
  check_outcome(outcome.var, data)
  check_time_vars(data, time.vars)
  tmp <- colnames(data)
  check_vars(covars, "covars", tmp, minlen=1)
  check_vars(anc.covars, "anc.covars", tmp)
  check_inclProb(inclProb.var, data)
  check_numeric(risk.time, "risk.time")
  check_numeric(risk.time0, "risk.time0", pos=0)
  check_numeric(nCntlPerCase, "nCntlPerCase")
  control <- check_control(control) 

  obj.list <- list(ncc.subset=ncc.subset, outcome.var=outcome.var, time.vars=time.vars, covars=covars, 
              anc.covars=anc.covars, tau=risk.time, inclProb.var=inclProb.var,
              nCntlPerCase=nCntlPerCase, tau0=risk.time0, min.per.miss=0.9, min.nrow=5, print=print,
              min.inclProb=min.inclProb, control=control)

  ret <- coxhaz_main(data, obj.list) 
  ret
}

coxhaz_phase1 <- function(data, obj.list) {

  covars      <- obj.list[["covars", exact=TRUE]]
  outcome.var <- obj.list$outcome.var
  time.vars   <- obj.list$time.vars
  form        <- as.formula(getCoxFormula(outcome.var, time.vars, covars))
  ret         <- coxph(form, data=data)
  ret

}

coxhaz_main <- function(data, obj.list) {
 
  tmp              <- cox_setupData(data, obj.list)
  data             <- tmp$data
  obj.list         <- tmp$obj.list
  tmp              <- NULL

  # Check the model columns, remove variables that need to be removed
  obj.list <- cox_checkCols(data, obj.list)

  # Phase 1
  fit.phase1 <- coxhaz_phase1(data, obj.list)

  ncc.subset       <- obj.list$ncc.subset
  covars           <- obj.list[["covars", exact=TRUE]]
  anc.covars       <- obj.list[["anc.covars", exact=TRUE]]
  incl.prob        <- data[, obj.list$inclProb.var, drop=TRUE]
  wgts             <- 1/incl.prob
  allcovars        <- c(covars, anc.covars)
  
  # Impute missing
  impute.vars <- getVarsToImpute(data, ncc.subset, covars, min.per.miss=obj.list$min.per.miss) 
  if (!length(impute.vars)) stop("ERROR: no covariates to impute")
  data2           <- data
  covarsForImpute <- allcovars[!(allcovars %in% impute.vars)]
  data2           <- predictMissingVars(data2, ncc.subset, wgts, impute.vars, covarsForImpute) 

  # Obtain Auxiliary Statistics (Influence Functions)
  aux <- cox_aux(data, data2, obj.list)

  # Calibrate weights (returned obj has length of subset)
  cal <- cox_calibrate(data2, aux, obj.list)

  # Fit cox hazards model (with non-imputed data)
  ret <- cox_hazModel(data, cal, obj.list) 

  ret <- coxhaz_setReturn(ret, obj.list, fit.phase1) 

  class(ret) <- "coxHazard"  

  ret

}

coxhaz_setReturn <- function(retobj, obj.list, fit.phase1) {

  vinfo <- obj.list$var.info.list

  tmp <- retobj[["with.calibration", exact=TRUE]]
  obj1 <- list(fit.phase2=tmp, fit.phase1=fit.phase1, var.info.list=vinfo)
  tmp <- retobj[["without.calibration", exact=TRUE]]
  obj2 <- list(fit.phase2=tmp, fit.phase1=fit.phase1, var.info.list=vinfo)
   
  list(with.calibration=obj1, without.calibration=obj2)
}

cox_hazModel <- function(data, cal, obj.list) {

  outcome   <- obj.list$outcome.var
  tvars     <- obj.list$time.vars
  covars    <- obj.list[["covars", exact=TRUE]]
  subset    <- obj.list$ncc.subset
  if (sum(subset) != length(cal)) stop("ERROR 1 in cox_hazModel")
  wgts      <- 1/data[, obj.list$inclProb.var, drop=TRUE]
  
  form    <- as.formula(getCoxFormula(outcome, tvars, covars)) 

  # No calibration
  fit0    <- coxph(form, data=data[subset, , drop=FALSE], weights=wgts[subset])

  # With calibration
  fit1    <- coxph(form, data=data[subset, , drop=FALSE], weights=cal)

  list(with.calibration=fit1, without.calibration=fit0)

}

cox_calibrate <- function(data, aux, obj.list) {

  subset  <- obj.list$ncc.subset
  wgts    <- 1/data[subset, obj.list$inclProb.var, drop=TRUE]
  control <- obj.list$control 
  ret <- rakecal(aux[subset, , drop=FALSE], wgts, colSums(aux), eta=TRUE, 
                 max_iter=control$maxit, EPS=control$eps, conv.type=control$conv)
  ret <- ret*wgts

  ret
}

cox_aux <- function(data, data2, obj.list) {
 
  yvar    <- obj.list$outcome.var
  outcome <- data[, yvar, drop=TRUE]
  tvars   <- obj.list$time.vars
  tau     <- obj.list$tau
  tau0    <- obj.list$tau0
  covars  <- obj.list[["covars", exact=TRUE]]
  subset  <- obj.list$ncc.subset
  wgts    <- 1/data[, obj.list$inclProb.var, drop=TRUE]
  control <- obj.list$control

  form    <- as.formula(getCoxFormula(yvar, tvars, covars)) 

  # Fit using imputed data
  fit     <- coxph(form, data=data2)
  DFBETA  <- residuals(fit, type="dfbeta")
  calX1   <- cbind(1, DFBETA)
  g1      <- rakecal(calX1[subset, , drop=FALSE], wgts[subset], colSums(calX1),
                     eta=TRUE, max_iter=control$maxit, EPS=control$eps, 
                     conv.type=control$conv)
  wgtcal1 <- g1*wgts[subset]

  # Fit using original data
  fit2    <- coxph(form, data=data[subset, , drop=FALSE], weights=wgtcal1)
  expb    <- predict(fit2, newdata=data2, type="risk")
  fuptime <- cox_followUpTime(data, tvars, tau0, tau) 
  if (length(expb) != length(fuptime)) stop("ERROR 1 in cox_aux")
  ret     <-  cbind(1, DFBETA, fuptime*expb)

  ret
}

cox_followUpTime <- function(data, time.vars, tau0, tau) {

  # For the age scale:
  # Taui = max[{min(xi, tau1) - max(bi, tau0)}, 0] 
  # For the time-on study scale:
  # Taui = max[{min(xi - bi, tau1) - tau0}, 0]

  if (length(time.vars) > 1) {
    vec1 <- pmin(data[, time.vars[2], drop=TRUE], tau)
    vec2 <- pmax(data[, time.vars[1], drop=TRUE], tau0)  
    ret  <- pmax(vec1 - vec2, 0)/(tau - tau0)
  } else {
    vec1 <- pmin(data[, time.vars[1], drop=TRUE], tau)
    ret  <- pmax(vec1 - tau0, 0)/(tau - tau0)
  }
  ret
}

cox_setupData <- function(data, obj.list) {

  ncc.subset       <- obj.list$ncc.subset
  outcome          <- obj.list$outcome.var
  time.vars        <- obj.list$time.vars
  covars           <- obj.list[["covars", exact=TRUE]]
  anc.covars       <- obj.list[["anc.covars", exact=TRUE]]
  inclProb.var     <- obj.list[["inclProb.var", exact=TRUE]]
  min.nrow         <- obj.list$min.nrow
  min.per.miss     <- obj.list$min.per.miss
  print            <- obj.list$print
  min.inclProb     <- obj.list$min.inclProb

  covars     <- unique(covars)
  anc.covars <- unique(anc.covars)

  var.info.list     <- NULL
  anc.var.info.list <- NULL

  # Check for overlapping variables
  check_overlap(outcome,    time.vars,  "outcome",    "time.vars")
  check_overlap(outcome,    covars,     "outcome",    "covars")
  check_overlap(outcome,    anc.covars, "outcome",    "anc.covars")
  check_overlap(anc.covars, covars,     "anc.covars", "covars")

  # Remove rows with missing values
  vars1      <- c(outcome, time.vars, inclProb.var)
  tmp        <- removeMiss(data, ncc.subset, vars1, covars, min.nrow=min.nrow, print=print) 
  data       <- tmp$data
  ncc.subset <- tmp$ncc.subset
  tmp        <- NULL

  # Dummy vars for categorical variables are needed
  if (!is.null(anc.covars)) {
    tmp <- checkAndAddDummy(data, anc.covars, print=print)
    if (length(tmp)) {
      data              <- tmp$data
      anc.covars        <- tmp$vars
      anc.var.info.list <- tmp$var.info.list
    } else {
      stop("ERROR: anc.covars are constant and have been removed")
    }
  } else {
    if (print) message("Since anc.covars were not specified, random noise will be used instead.")

    # Add random noise
    anc.covars         <- getRandomVariable("ANC", len=12)
    data[, anc.covars] <- runif(nrow(data))
  }
 
  tmp <- checkAndAddDummy(data, covars, print=print)
  if (length(tmp)) {
    data          <- tmp$data
    covars        <- tmp$vars
    var.info.list <- tmp$var.info.list
  }

  if (is.null(inclProb.var)) {
    etime <- getEventTime(data, time.vars)
    tmp   <- ncc.sample.prob(etime, data[, outcome, drop=TRUE], obj.list$nCntlPerCase)
    if (nrow(tmp) != nrow(data)) stop("ERROR calling ncc.sample.prob")
    if (is.null(inclProb.var)) {
      inclProb.var         <- getRandomVariable("INCLPROB", len=12)
      data[, inclProb.var] <- tmp[, 2]
    }
  }
  tmp <- data[, inclProb.var, drop=TRUE] < min.inclProb
  tmp[is.na(tmp)] <- FALSE
  if (any(tmp)) {
    if (print) message("Some inclusion probabilities were less than min.inclProb")
    data[tmp, inclProb.var] <- min.inclProb
  }

  # Only keep variables we need 
  vars <- c(outcome, time.vars, covars, anc.covars, inclProb.var)
  data <- data[, vars, drop=FALSE]

  obj.list$covars            <- covars
  obj.list$anc.covars        <- anc.covars
  obj.list$inclProb.var      <- inclProb.var
  obj.list$var.info.list     <- var.info.list
  obj.list$anc.var.info.list <- anc.var.info.list
  obj.list$ncc.subset        <- ncc.subset

  list(data=data, obj.list=obj.list)

}

cox_checkCols <- function(data, obj.list) {

  vars <- obj.list[["covars", exact=TRUE]]
  rem  <- getColsToRemove(data[, vars, drop=FALSE], add.int=1)
  if (!length(rem)) return(obj.list)

  var.info.list <- obj.list$var.info.list  
  var.info.list <- updateVarInfoList(var.info.list, rem)

  tmp           <- !(vars %in% rem)
  vars          <- vars[tmp]
  if (!length(vars)) stop("ERROR: all covars have been removed")

  obj.list$covars        <- vars
  obj.list$var.info.list <- var.info.list

  obj.list
}

bhaz.wgt = function(coxph_object, wgt = NULL){
  
  coxdetail_object = coxph.detail(coxph_object, riskmat = T)
  
  Z = model.matrix(coxph_object)
  Zbeta = Z %*% coxph_object$coefficients
  
  risk_mat = coxdetail_object$riskmat
  if(is.null(wgt)) wgt = coxph_object$weight
  YwexpbZ = sweep(risk_mat, 1, wgt*exp(Zbeta), `*`)
  
  dNt = matrix(0, nrow(risk_mat), ncol(risk_mat))
  evnt = apply(risk_mat, 1, function(x) { ifelse(all(x==0), 0, max(which(x == 1))) } )
  dNt[cbind(1:nrow(risk_mat), evnt)] = 1
  dNt = sweep(dNt, 1, coxph_object$y[,ncol(coxph_object$y)], `*`)
  
  Gt = colSums(YwexpbZ)
  H0t = colSums(dNt)/Gt
  
  return(H0t)
}

pure.risk.zst <- function(bcoef, bHaz, zstar){
  result <- as.vector(1 - exp(-bHaz*exp(bcoef %*% zstar)))
  result
}

coxHazardPureRisk <- function(coxHazObj, newdata, risk.time, risk.time0=0,
                              new.risk.col="pureRisk") {

  check_cox_fit(coxHazObj)
  check_data(newdata, name="newdata")
  check_numeric(risk.time, "risk.time")
  check_numeric(risk.time0, "risk.time0", pos=0)

  tmp <- coxHazardPureRisk_main(coxHazObj, newdata, risk.time, risk.time0=risk.time0)
  ret <- newdata
  ret[, new.risk.col] <- tmp
  ret

}

coxHazardPureRisk_main <- function(coxHazObj, newdata, risk.time, risk.time0) {

  var.info.list   <- coxHazObj[["var.info.list", exact=TRUE]]
  covars          <- names(var.info.list)
  check_varnames(covars, colnames(newdata), data.name="newdata")
  covars.list0    <- var.info.list

  tmp <- checkAndAddDummy(newdata, covars, print=0, drop=0)
  if (!length(tmp)) stop("ERROR 1")
  newdata     <- tmp$data
  covars      <- tmp$vars
  covars.list <- tmp$var.info.list  

  # Check for consistent variable types and categories
  checkVarInfo2(covars, covars.list0, covars.list)
  covars   <- getDvarsFromInfo(var.info.list)

  bhaz1    <- bhaz.wgt(coxHazObj$fit.phase1, wgt=1)
  bhazt    <- as.numeric(names(bhaz1))
  bhaz2    <- bhaz.wgt(coxHazObj$fit.phase2)
  bHaz_ph2 <- sum(bhaz2[bhazt<risk.time])-sum(bhaz2[bhazt<risk.time0])

  # Get coefficients
  betavec <- coxHazObj$fit.phase2$coefficients
  vars    <- names(betavec)
  if (!all(vars %in% colnames(newdata))) stop("ERROR 2")

  # Order data according to vars
  newdata <- newdata[, vars, drop=FALSE]

  # Get unique covariate patterns
  xids  <- myPasteCols(newdata, sep=":")
  uids  <- unique(xids)
  rows  <- match(xids, uids)
  tmp   <- !duplicated(xids)
  data2 <- newdata[tmp, , drop=FALSE] 
  gc()
  
  missing <- rowSums(!is.finite(as.matrix(data2)))
  nr      <- nrow(data2)
  ret.est <- rep(NA, nr)
  for (i in 1:nr) {
    if (!missing[i]) {
      zstar      <- unlist(data2[i, , drop=TRUE])
      ret.est[i] <- pure.risk.zst(betavec, bHaz_ph2, zstar)
    }
  }
  est <- ret.est[rows] 

  est

}

print.coxHazard <- function(x, ...) {

  cat("With Calibration:\n")
  print(x$with.calibration$fit.phase2)
  cat("\n")
  cat("Without Calibration:\n")
  print(x$without.calibration$fit.phase2)

  NULL
}