diagnosticTests <- function(data, ncc.subset, outcome.var, time.vars, 
                            timeDep.covars, timeIndep.covars, nCntlPerCase=1,
                            inclProb.var=NULL, start.time=0, print=TRUE, plot=TRUE,
                            min.inclProb=1e-6) {

  check_data(data)
  check_subset(ncc.subset, "ncc.subset", nrow(data)) 
  check_outcome(outcome.var, data)
  check_time_vars(data, time.vars)
  tmp <- colnames(data)
  check_vars(timeDep.covars, "timeDep.covars", tmp)
  check_vars(timeIndep.covars, "timeIndep.covars", tmp)
  check_inclProb(inclProb.var, data)

  obj.list <- list(ncc.subset=ncc.subset, outcome.var=outcome.var, time.vars=time.vars, 
                   timeDep.covars=timeDep.covars, timeIndep.covars=timeIndep.covars, 
                   inclProb.var=inclProb.var, start.time=start.time, nCntlPerCase=nCntlPerCase,
                   min.inclProb=min.inclProb, min.nrow=5, DIAGNOSTIC=1, print=print, plot=plot)

  ret <- diagnosticTests_main(data, obj.list) 

}

diagnosticTests_main <- function(data, obj.list) {

  tmp        <- add_setupData(data, obj.list)
  data       <- tmp$data
  obj.list   <- tmp$obj.list
  tmp        <- NULL
  plot       <- obj.list$plot

  propHaz.test <- testForPropHazard(data, obj.list) 
  if (plot) plot(propHaz.test, resid=TRUE, main="Scaled Schoenfeld Residuals")
  addHaz.test <- testForAddHazard(data, obj.list)
  if (plot) plotAalen(addHaz.test, plot.vars=NULL)
  tests        <- addHaz.test$obs.testBeqC
  pvals        <- addHaz.test$pval.testBeqC
  ks           <- cbind(tests, pvals)
  rownames(ks) <- names(tests)
  colnames(ks) <- c("Kolmogorov-Smirnov test", "p-value H_0:constant effect")

  ret <- list(propHaz.test=propHaz.test$table, addHaz.test=ks)
}

testForPropHazard <- function(data, obj.list) {

  outcome.var      <- obj.list$outcome.var 
  time.vars        <- obj.list$time.vars 
  timeDep.covars   <- obj.list$timeDep.covars
  timeIndep.covars <- obj.list$timeIndep.covars
  inclProb.var     <- obj.list$inclProb.var
  wgts             <- 1/data[, inclProb.var, drop=TRUE]
  
  form   <- as.formula(getCoxFormula(outcome.var, time.vars, c(timeDep.covars, timeIndep.covars)))
  fitcox <- coxph(form, data=data, weights= 1/wgts)
  ret    <- cox.zph(fitcox, transform="identity")

  ret
}

testForAddHazard <- function(data, obj.list) {

  outcome.var      <- obj.list$outcome.var 
  time.vars        <- obj.list$time.vars 
  timeDep.covars   <- obj.list$timeDep.covars
  timeIndep.covars <- obj.list$timeIndep.covars
  subset           <- obj.list$ncc.subset
  start.time       <- obj.list$start.time

  form <- getAalenFormula(outcome.var, time.vars, timeDep.covars, timeIndep.covars)
  form <- as.formula(form)
  fit  <- aalen(form, data=data[subset, , drop=FALSE], start.time=start.time)

  fit

}

plotAalen <- function(obj, plot.vars=NULL) {

  cum  <- obj$cum
  nms  <- colnames(cum)
  time <- cum[, 1, drop=TRUE]
  n    <- length(plot.vars)
  if (!n) plot.vars <- nms[-1]
  tmp  <- plot.vars %in% nms
  vars <- plot.vars[tmp]
  n    <- length(plot.vars)
  if (n != length(plot.vars)) {
    tmp  <- !(plot.vars %in% vars)
    miss <- paste0(plot.vars[tmp], collapse=", ")
    msg  <- paste0("The variables(s) ", miss, " were not found and will not be plotted")
    warning(msg)
  }
  if (n) {
    for (i in 1:n) {
      var <- vars[i]
      plot(x=time, y=cum[, var, drop=TRUE], type = "s", 
           main=var, xlab = "Time", ylab = "Cumulative Estimate")
      abline(h=0, lty=3)
    }
  }
  NULL
}

getAalenFormula <- function(yvar, timevar, timeDep.covars, timeIndep.covars) {

  if (length(timevar) == 1) {
    str0 <- paste0("Surv(", timevar, ", ", yvar, ")")
  } else {
    str0 <- paste0("Surv(", timevar[1], ", ", timevar[2], ", ", yvar, ")")
  }

  if (length(timeIndep.covars)) timeIndep.covars <- paste0("const(", timeIndep.covars, ")")
  covars <- c(timeDep.covars, timeIndep.covars)

  if (length(covars)) {
    str <- paste0(covars, collapse=" + ")
    ret <- paste0(str0, " ~ ", str) 
  } else {
    ret <- paste0(str0, " ~ 1")
  }
  ret
}

