check_add_fit <- function(x, name="fit") {

  if (!is.list(x)) stop(paste0("ERROR: ", name, " must be a list"))
  valid <- c("est", "Bt.hat", "se", "infl",
             "infl.omg", "sqrt.prob", "ind.omega", "jcov.byp",
             "timeDep.var.info.list", "timeIndep.var.info.list")
  nms  <- names(x)
  tmp  <- !(valid %in% nms)
  miss <- valid[tmp]
  if (length(miss)) {
    msg <- paste0(miss, collapse=", ")
    msg <- paste0("ERROR: ", name, " is missing object(s) ", msg)
    stop(msg)
  } 
  
  NULL

}

check_cox_fit <- function(x, name="fit") {

  if (!is.list(x)) stop(paste0("ERROR: ", name, " must be a list"))
  valid <- c("fit.phase2", "fit.phase1", "var.info.list")
  nms  <- names(x)
  tmp  <- !(valid %in% nms)
  miss <- valid[tmp]
  if (length(miss)) {
    msg <- paste0(miss, collapse=", ")
    msg <- paste0("ERROR: ", name, " is missing object(s) ", msg)
    stop(msg)
  } 
  
  NULL

}

check_control <- function(x, name="control") {

  if (is.null(x)) x <- list()
  if (!is.list(x)) stop(paste0("ERROR: ", name, " must be a list"))
  valid <- c("maxit", "eps", "conv")
  def   <- list(1000, 1e-8, "absolute")
  x     <- default.list(x, valid, def)
  check_integer(x$maxit, "maxit")
  check_numeric(x$eps, "eps")
  x$conv <- substring(removeWhiteSpace(tolower(x$conv)), 1, 3)
  check_string(x$conv, "conv", c("abs", "rel"))

  x
}

check_string <- function(x, name, valid) {

  len <- length(x)
  if (!is.character(x) || (len != 1)) {
    str <- paste0(valid, collapse=", ")
    stop(paste0("ERROR: ", name, " must be one of ", str))
  }
  if (!(x %in% valid)) {
    str <- paste0(valid, collapse=", ")
    stop(paste0("ERROR: ", name, " must be one of ", str))
  }
  NULL
}

check_aux.type <- function(x, name="aux.type") {

  valid <- c("log-relative", "baseline")
  if (!isString(x)) stop("ERROR: ", name, " must be ", valid[1], " or ", valid[2])
  x <- tolower(removeWhiteSpace(x))
  if (!(x %in% valid)) stop("ERROR: ", name, " must be ", valid[1], " or ", valid[2])
  x

}

check_time.scale <- function(x, name="time.scale") {

  valid <- c("age", "time-on-study")
  if (!isString(x)) stop("ERROR: ", name, " must be ", valid[1], " or ", valid[2])
  x <- tolower(removeWhiteSpace(x))
  if (!(x %in% valid)) stop("ERROR: ", name, " must be ", valid[1], " or ", valid[2])
  x

}

check_overlap <- function(set1, set2, name1, name2) {

  if (length(intersect(set1, set2))) {
    msg <- paste0("ERROR: ", name1, " and ", name2, " cannot have variables in common")
    stop(msg)
  }
  NULL
}

check_outcome <- function(x, data, name="outcome") {

  check_var(x, name, colnames(data), exist=1) 
  check_outcome_vec(data[, x, drop=TRUE], name)
  NULL
}

check_outcome_vec <- function(x, name) {

  type <- getVarType(x, only.finite=1)
  if (type != "binary") stop(paste0("ERROR: ", name, " must be a binary variable"))
  NULL
 
}

check_time_vars <- function(data, vars, name="time.vars") {

  check_vars(vars, name, colnames(data), maxlen=2, minlen=1)
  n  <- length(vars)
  x1 <- data[, vars[1], drop=TRUE]
  check_numeric(x1, vars[1], pos=1, len=0) 
  if (n > 1) {
    x2 <- data[, vars[2], drop=TRUE]
    check_numeric(x2, vars[2], pos=1, len=0)
    if (vars[1] == vars[2]) stop(paste("ERROR: duplicate variable name in ", name))
    tmp <- x1 > x2
    tmp[is.na(tmp)] <- FALSE
    if (any(tmp)) {
      msg <- paste0("ERROR: there are subjects with ", vars[1], " > ", vars[2])
      stop(msg)
    } 
  }
  NULL
}

check_inclProb <- function(var, data, name="inclProb.var") {
 
  cx <- colnames(data)
  check_var(var, name, cx, exist=0)  
  if (length(var)) {
    x <- data[, var, drop=TRUE]
    check_numeric(x, name, pos=1, len=0, maxval=1) 
  }
  NULL
}

check_data <- function(x, name="data") {

  if (!is.data.frame(x)) stop(paste0("ERROR: ", name, " must be a data frame"))
  if (!nrow(x)) stop(paste0("ERROR: ", name, " contains no rows"))
  if (!ncol(x)) stop(paste0("ERROR: ", name, " contains no columns"))
  NULL

}

check_subset <- function(x, name, len) {

  if (len && (len != length(x))) stop(paste0("ERROR: ", name, " must have length ", len))
  if (!is.logical(x) || !is.vector(x)) stop(paste0("ERROR: ", name, " must be a logical vector"))
  NULL

}

check_aux.stat.vars <- function(x, xvars, zvars) {

  nm  <- "aux.vars"
  nmx <- "timeDep.covars"
  nmz <- "timeIndep.covars"

  n <- length(x)
  if (!n) return(NULL)
  if (!is.character(x)) stop(paste0("ERROR: ", nm, " must be NULL or a character vector of column names"))
  valid <- c(xvars, zvars)
  tmp   <- !(x %in% valid)
  if (any(tmp)) {
    msg <- paste0("ERROR: ", nm, " must be variables listed in ", nmx, " or ", nmz)
    stop(msg)
  }
  NULL

}

check_var <- function(x, name, valid, exist=1) {

  if (!exist) {
    if (!length(x)) return(NULL)
  }
  if (!isString(x)) stop(paste0("ERROR: ", name, " must be a column name in the data"))
  if (!(x %in% valid)) stop(paste0("ERROR: ", name, " is not a valid column name in the data"))
  NULL

}

check_vars <- function(x, name, valid, maxlen=0, minlen=0) {

  len <- length(x)
  if (!len) return(NULL)
  if (maxlen && (len > maxlen)) stop(paste0("ERROR: ", name, " must have length <= ", maxlen))
  if (minlen && (len < minlen)) stop(paste0("ERROR: ", name, " must have length >= ", minlen))

  if (!is.character(x)) stop(paste0("ERROR: ", name, " must be column names in the data"))
  tmp <- !(x %in% valid)
  if (any(tmp)) {
    miss <- x[tmp]
    str  <- paste0(miss, collapse=", ")  
    stop(paste0("ERROR: ", name, " contains the invalid column name(s) ", str))
  }
  NULL

}

check_varnames <- function(varnames, data.cols, data.name="data") {

  if (!length(varnames)) return(NULL)
  tmp <- !(varnames %in% data.cols)
  if (any(tmp)) {
    miss <- paste0(varnames[tmp], collapse=", ")
    msg  <- paste0("ERROR: ", data.name, " does not contain column(s) ", miss)
    stop(msg)
  }
  NULL
}

check_numeric <- function(x, name, pos=1, len=1, maxval=Inf) {

  if (!is.numeric(x)) stop(paste0("ERROR: ", name, " must be numeric"))
  if (len && (len != length(x))) stop(paste0("ERROR: ", name, " must have length ", len))
  if (pos) {
    tmp <- x <= 0
    tmp[is.na(tmp)] <- FALSE
    if (any(tmp)) {
      if (len == 1) {
        stop(paste0("ERROR: ", name, " must be a positive value")) 
      } else {
        stop(paste0("ERROR: ", name, " must have positive values")) 
      }
    }  
  }
  if (is.finite(maxval)) {
    tmp <- x > maxval
    tmp[is.na(tmp)] <- FALSE
    if (any(tmp)) {
      stop(paste0("ERROR: ", name, " must be <= ", maxval)) 
    }
  } 

  NULL
}

check_integer <- function(x, name, len=1, minval=1, maxval=Inf) {

  if (len && (len != length(x))) stop(paste0("ERROR: ", name, " must have length ", len))
  if (!is.numeric(x)) stop(paste0("ERROR: ", name, " must be integer"))
  tmp <- (x != floor(x))
  if (any(tmp)) stop(paste0("ERROR: ", name, " must be integer"))  
 
  if (is.finite(minval)) {
    tmp <- x < minval
    tmp[is.na(tmp)] <- FALSE
    if (any(tmp)) {
      stop(paste0("ERROR: ", name, " must be >= ", minval))  
    }  
  }
  if (is.finite(maxval)) {
    tmp <- x > maxval
    tmp[is.na(tmp)] <- FALSE
    if (any(tmp)) {
      stop(paste0("ERROR: ", name, " must be <= ", maxval)) 
    }
  } 

  NULL
}


isString <- function(x) {
  ret <- (length(x) == 1) && is.character(x)
  ret
}

predictMissingVars <- function(data, subset, wgts, varsToImpute, allvars) {

  if (!length(varsToImpute)) return(data)  
  if (!length(allvars)) stop("ERROR 1")  

  allvars      <- unique(allvars)
  varsToImpute <- unique(varsToImpute)
  ret          <- data
  for (var in varsToImpute) {
    tmp  <- !(allvars == var)
    vv   <- allvars[tmp]
    form <- as.formula(getGlmFormula(var, vv))
    type <- getVarType(data[subset, var, drop=TRUE])

    if (type == "binary") {
      ret <- predictMissingVar(ret, subset, form, "binomial", wgts, var) 
    } else if (type == "cont") {
      ret <- predictMissingVar(ret, subset, form, "gaussian", wgts, var) 
    } else {
      stop("ERROR 1")
    }
  }
  ret
}

predictMissingVar <- function(data, subset, form, family, wgts, var) {

  ret           <- data
  fit           <- suppressWarnings(glm(form, family=family, data=data[subset, , drop=FALSE], weights=wgts[subset]))
  vec           <- predict(fit, type="response", newdata=data)
  if (length(vec) != nrow(data)) stop("ERROR 1")
  ret[, var]    <- vec

  ret
}

getVarType <- function(vec, only.finite=0) {

  ret <- NULL
  if (is.character(vec) || is.factor(vec)) {
    ret <- "cat"
  } else if (is.numeric(vec)) {
    if (only.finite) vec <- vec[is.finite(vec)]  
    if (all(vec %in% 0:1)) {
      ret <- "binary"
    } else {
      ret <- "cont"
    }
  } else {
    stop("ERROR 1")
  }  
  ret
}

getVarsToImpute <- function(data, ncc.subset, covars, min.per.miss=0.9) {

  if (!length(covars)) return(NULL)
  ret  <- NULL
  keep <- !ncc.subset
  for (v in covars) {
    x    <- data[keep, v, drop=TRUE]
    type <- getVarType(x) 
    if (type == "cat") {
      tmp <- is.na(x)
    } else {
      tmp <- !is.finite(x) 
    }
    if (sum(tmp) >= min.per.miss) ret <- c(ret, v) 
  }
  ret
}

checkAndAddDummy <- function(data, vars, print=1, drop=1) {

  # drop: 0 or 1 For model fitting set drop to 1 (default)
  #              For put risk, set drop to 0

  if (!length(vars)) return(NULL)

  retvars  <- NULL
  dropvars <- NULL
  varinfo  <- list()
  for (v in vars) {
    vec   <- data[, v, drop=TRUE]
    uvec  <- sort(unique(vec))
    nuvec <- length(uvec) 
    if (nuvec > drop) {
      type <- getVarType(vec)
      if (type == "cat") {
        # Normalize
        vec   <- normCharVec(vec)
        uvec  <- sort(unique(vec))
        nuvec <- length(uvec)

        # First category will be the baseline
        if (drop) {
          cats  <- uvec[-1]
        } else {
          cats  <- uvec
        }
        cats0 <- uvec
        ncats <- length(cats)
        dvars <- paste0(v, "_", cats)
        for (i in 1:ncats) {
          data[, dvars[i]] <- as.numeric(vec %in% cats[i])
        }
        retvars      <- c(retvars, dvars)
      } else {
        retvars      <- c(retvars, v)
        dvars        <- v 
        cats0        <- NULL
      }    
      varinfo[[v]] <- list(dvars=dvars, vartype=type, cats=cats0)
    } else {
      # Drop this variable
      if (print) message(paste0("Variable ", v, " has been dropped from the model."))
      dropvars <- c(dropvars, v)
    }
  }
  if (is.null(retvars)) return(NULL)  

  list(data=data, vars=retvars, drop.vars=dropvars, var.info.list=varinfo)

}

getGlmFormula <- function(yvar, covars) {

  if (length(covars)) {
    str <- paste0(covars, collapse=" + ")
    ret <- paste0(yvar, " ~ ", str) 
  } else {
    ret <- paste0(yvar, " ~ 1")
  }
  ret
}

getCoxFormula <- function(yvar, timevar, covars) {

  if (length(timevar) == 1) {
    str0 <- paste0("Surv(", timevar, ", ", yvar, ")")
  } else {
    str0 <- paste0("Surv(", timevar[1], ", ", timevar[2], ", ", yvar, ")")
  }
  if (length(covars)) {
    str <- paste0(covars, collapse=" + ")
    ret <- paste0(str0, " ~ ", str) 
  } else {
    ret <- paste0(str0, " ~ 1")
  }
  ret
}

getRandomVariable <- function(prefix, len=12) {

  cc  <- c(letters, as.character(0:9))
  vec <- sample(cc, len, replace=TRUE)
  str <- paste0(vec, collapse="")
  ret <- paste0(prefix, ".", str)
  ret
}

getEventTime <- function(data, time.vars) {

  nt <- length(time.vars)
  if (nt == 1) {
    ret <- data[, time.vars, drop=TRUE]
  } else if (nt == 2) {
    ret <- data[, time.vars[2], drop=TRUE] - data[, time.vars[1], drop=TRUE]
  } else {
    stop("ERROR 1")
  }
  ret
}

normCharVec <- function(vec, replace=".") {

  ret <- gsub("[^[:alnum:]]", replace, vec, perl=TRUE)
  ret
}

getNonMissing <- function(vec) {

  type <- getVarType(vec, only.finite=0) 
  if (type == "cat") {
    ret <- !is.na(vec)
  } else {
    ret <- is.finite(vec)
  }
  ret
}

removeMiss <- function(data, ncc.subset, vars1, vars2, min.nrow=5, print=1) {

  nr0 <- nrow(data)

  # Remove rows with missing values in vars1
  n1 <- length(vars1)
  if (n1) {
    tmp <- getNonMissing(data[, vars1[1], drop=TRUE])
    if (n1 < 1) {
      for (v in vars1[-1]) tmp <- tmp & getNonMissing(data[, v, drop=TRUE])
    }
    data       <- data[tmp, , drop=FALSE]
    ncc.subset <- ncc.subset[tmp]
  }

  # Remove missing from phase 2 covariates
  if (length(vars2)) {
    tmp <- rep(TRUE, nrow(data))
    for (v in vars2) {
      tmp <- ncc.subset & !getNonMissing(data[, v, drop=TRUE])
      if (any(tmp)) {
        data       <- data[!tmp, , drop=FALSE]
        ncc.subset <- ncc.subset[!tmp]
      }
    } 
  }
  nr <- nrow(data)
  m  <- nr0 - nr
  if (nr < min.nrow) stop("ERROR: after removing missing values, the data contains too few rows")
  if (print && m) {
    if (m > 1) {
      message(paste0("NOTE: ", nr0-nr, " subjects have been removed due to missing values"))
    } else {
      message(paste0("NOTE: 1 subject has been removed due to missing values"))
    }
  }

  list(data=data, ncc.subset=ncc.subset)

}

removeWhiteSpace <- function(str, leading=1, trailing=1) {

  if ((leading) && (trailing)) {
    ret <- gsub("^\\s+|\\s+$", "", str, perl=TRUE)
  } else if (leading) {
    ret <- gsub("^\\s+", "", str, perl=TRUE)
  } else if (trailing) {
    ret <- gsub("\\s+$", "", str, perl=TRUE)
  } else {
    ret <- str
  }

  ret

}

default.list <- function(inList, names, default, error=NULL,
                         checkList=NULL) {

  # inList      List
  # names       Vector of names of items in inList
  # default     List of default values to assign if a name is not found
  #             The order of default must be the same as in names.
  # error       Vector of TRUE/FALSE if it is an error not to have the
  #             name in the list. 
  #             The default is NULL
  # checkList   List of valid values for each name.
  #             Use NA to skip a list element.
  #             The default is NULL

  n1 <- length(names)
  n2 <- length(default)
  if (n1 != n2) stop("ERROR: in calling default.list")

  if (is.null(error)) {
    error <- rep(0, times=n1)
  } else if (n1 != length(error)) {
    stop("ERROR: in calling default.list")
  }

  if (!is.null(checkList)) {
    if (n1 != length(checkList)) stop("ERROR: in calling default.list")
    checkFlag <- 1
  } else {
    checkFlag <- 0
  } 

  if (is.null(inList)) inList <- list()

  listNames <- names(inList)
  for (i in 1:n1) {
    if (!(names[i] %in% listNames)) {
      if (!error[i]) {
        inList[[names[i]]] <- default[[i]]
      } else {
        temp <- paste("ERROR: the name ", names[i], " was not found", sep="")
        stop(temp)
      }
    } else if (checkFlag) {
      temp <- checkList[[i]]
      if (!all(is.na(temp))) {
        if (!all(inList[[names[i]]] %in% checkList[[i]])) {
          temp <- paste("ERROR: the name '", names[i], 
                      "' has an invalid value", sep="")
          stop(temp)
        }
      }
    }
  }

  inList

} 

checkVarInfo2 <- function(covars, covars.list0, covars.list) {

  for (v in covars) {
    tlist0 <- covars.list0[[v, exact=TRUE]]
    if (is.null(tlist0)) stop("ERROR 2")
    tlist  <- covars.list[[v, exact=TRUE]]
    if (is.null(tlist)) stop("ERROR 3")
    type0 <- tlist0$vartype
    type  <- tlist$vartype
    if ((type == "cat") || (type0 == "cat")) {
      if (type != type0) stop(paste0("ERROR: incorrect variable type for ", v))
      cats  <- tlist$cats
      cats0 <- tlist0$cats
      tmp   <- !(cats %in% cats0)
      if (any(tmp)) {
        str <- paste0(cats[tmp], collapse=", ")
        msg <- paste0("ERROR: categories ", str, " for variable ", v, 
                      " not found in fitted model")
        stop(msg)  
      }
    }
  }
  NULL
}

getDvarsFromInfo <- function(infolist) {

  ret <- NULL
  for (i in 1:length(infolist)) {
    tmp <- infolist[[i]] 
    if (length(tmp)) {
      dvars <- tmp[["dvars", exact=TRUE]]
      if (length(dvars)) ret <- c(ret, dvars)
    }
  }
  ret
}

myPasteCols <- function(x, sep="*") {

  nc  <- ncol(x)
  ret <- x[, 1, drop=TRUE]
  if (nc > 1) {
    for (i in 2:nc) ret <- paste(ret, x[, i, drop=TRUE], sep=sep)
  }
  ret
 
}

getColsToRemove <- function(data, add.int=1, print=1) {

  rem <- NULL
  x   <- as.matrix(data)
  if (add.int) {
    cx  <- colnames(x)
    x   <- cbind(1, x)
    colnames(x) <- c("(intercept)", cx)
  }
  tmp <- rowSums(!is.finite(x)) == 0
  x   <- x[tmp, , drop=FALSE]
  tmp <- caret::findLinearCombos(x)
  rem.cols <- tmp[["remove", exact=TRUE]]
  if (length(rem.cols)) {
    cx  <- colnames(x)
    rem <- cx[rem.cols]
  }
  if (print && length(rem)) {
    msg <- paste0(rem, collapse=", ")
    msg <- paste0("The variable(s) ", msg, " have been removed from the model due to linear dependencies")
    warning(msg)
  }
  rem
}

updateVarInfoList <- function(vinfo, rem.vars) {

  n <- length(vinfo)
  if (!n) return(vinfo)
  nms <- names(vinfo)
  for (i in 1:length(nms)) {
    var <- nms[i]
    if (var %in% rem.vars) {
      vinfo[[var]] <- NULL
      next
    }
    dvars <- vinfo[[var]]$dvars
    tmp   <- dvars %in% rem.vars
    if (any(tmp)) {
      dvars <- dvars[!tmp]
      if (length(dvars)) {
        vinfo[[var]]$dvars <- dvars
      } else {
        vinfo[[var]] <- NULL
      }
    } 
  }
  vinfo

}
