\name{addHazard}
\alias{addHazard}
\title{Fit Additive Hazards Model from Nested Case-control Samples (NCC) using weight calibration}
\usage{
addHazard(data, ncc.subset, outcome.var, time.vars, timeDep.covars, timeIndep.covars, 
          anc.covars, risk.time, risk.time0=0, nCntlPerCase=1,
          inclProb.var=NULL, nrisk.var=NULL, 
          print=1, min.inclProb=1e-6, control=NULL) 
}
\arguments{
\item{data}{Data frame containing all the data for analysis.}
\item{ncc.subset}{Logical vector giving the subjects in the nested case-control sample.
                  The length must be equal to the number of rows in \code{data}. }
\item{outcome.var}{Binary outcome variable in \code{data} (1 = case, 0 = non-case).}
\item{time.vars}{One or two time-to-event variables in \code{data}.}
\item{timeDep.covars}{Character vector of covariates with time-varying effects.}
\item{timeIndep.covars}{Character vector of covariates with time-invariant effects.}
\item{anc.covars}{NULL or a character vector of ancillary covariates used to impute missing
                  values of \code{timeDep.covars} and \code{timeIndep.covars}
                  for subjects not in the nested case-control sample. 
                  If NULL, then random noise will be used for the imputation (see details).}
\item{risk.time}{Projection time for pure risk.}
\item{risk.time0}{Initial projection time for pure risk. Only used \code{time.vars} contains
                  two time variables. The default is 0.}
\item{nCntlPerCase}{Number of controls per case. The default is 1.}
\item{inclProb.var}{NULL or a variable in \code{data} giving the probability of being included in 
                    the nested case-control sample. 
                    This can be computed using the function \code{multipleNCC::KMprob}.
                    If NULL, then it is internally computed based on \code{outcome.var},
                    \code{time.vars}, and \code{nCntlPerCase}. The default is NULL.}
\item{nrisk.var}{NULL or a variable in \code{data} giving the number of at-risk subjects at each time.
                 If NULL, thenit is internally computed based on \code{outcome.var},
                    \code{time.vars}, and \code{nCntlPerCase}. The default is NULL.}
\item{print}{0 or 1 to print information. The default is 1.}
\item{min.inclProb}{Positive value \code{<= 1} giving the minimum inclusion probability for 
                    being in the nested case-control sample. Any inclusion probability 
                    less than \code{min.inclProb} will be set to \code{min.inclProb}.
                    This cannot be 0, since weights will be formed by taking the reciprocals
                    of the inclusion probabilities.
                    The default is 1e-6.}
\item{control}{See \code{\link{control}}.}
}
\value{
  A list containing two lists: \code{with.calibration} and \code{without.calibration}. 
  Each sublist contains the parameter estimates, standard errors, and other
  objects needed to estimate pure risk using \code{\link{addHazardPureRisk}}.
}
\description{
  This function fits the additive hazards model with time-varying coefficients and 
  time-invariant coefficients from a nested case-control study with or without weight calibration.
}
\details{
  \bold{NOTE:} If \code{anc.covars = NULL}, then a seed must be set in order to reproduce
              the results. See Step 1 below.\cr

  The algorithm is as follows: \cr

  \bold{Step 1:} Obtain predictions for missing data \cr
  For any variable in \code{timeDep.covars} and \code{timeIndep.covars} with missing
     data from phase 1,
    fit a weighted generalized linear model among the nested case-control subjects (phase 2)
    with that variable as the outcome adjusting for the 
    other covariates in \code{timeDep.covars}, \code{timeIndep.covars} and \code{anc.covars}.
    The weights in the model are the reciprocals of the inclusion probabilities (\code{inclProb.var}).
    If \code{anc.covars = NULL}, then \code{runif(nrow(data))} will be used as an ancillary 
    covariate. \cr

  \bold{Step 2:} Create auxiliary statistics (influence functions) \cr

  \bold{Step 3:} Calibrate the design weights \cr

  \bold{Step 4:} Fit the additive hazards model with and without weight calibration. \cr

}
\seealso{ \code{\link{coxHazard}}, \code{\link{diagnosticTests}} }
\keyword{model, addHazard}

\examples{

  data(sample_data, package="WeightCalibSurvival")

  # Set the input arguments
  ncc.subset       <- sample_data[, "ind.ph2"] %in% 1
  outcome.var      <- "ind.fail"
  time.vars        <- "eventime"
  timeDep.covars   <- c("X1", "X2")
  timeIndep.covars <- c("Z1", "Z2")
  anc.covars       <- "U"
  risk.time        <- 8
  inclProb.var     <- "incl.prob"
  nrisk.var        <- "nrisk"

  addHazard(sample_data, ncc.subset, outcome.var, time.vars, timeDep.covars, timeIndep.covars, 
            anc.covars, risk.time, inclProb.var=inclProb.var, nrisk.var=nrisk.var)
}

\author{
  Yei Eun Shin \email{syeeun@gmail.com}
}

\references{
  Shin YE, Pfeiffer RM Graubard BI, Gail MH. Weight calibration to
  improve efficiency for estimating pure risks from
  the additive hazards model with the nested
  case-control design. Biometrics. 2020;1-13.
  https://doi.org/10.1111/biom.13413
}


