\name{coxHazard}
\alias{coxHazard}
\title{Fit Cox Model from Nested Case-control Samples (NCC) using weight calibration}
\usage{
coxHazard(data, ncc.subset, outcome.var, time.vars, covars, 
          anc.covars, risk.time, risk.time0=0, nCntlPerCase=1,
          inclProb.var=NULL, print=1, min.inclProb=1e-6, control=NULL) 
}
\arguments{
\item{data}{Data frame containing all the data for analysis.}
\item{ncc.subset}{Logical vector giving the subjects in the nested case-control sample.
                  The length must be equal to the number of rows in \code{data}. }
\item{outcome.var}{Binary outcome variable in \code{data} (1 = case, 0 = non-case).}
\item{time.vars}{One or two time-to-event variables in \code{data}.}
\item{covars}{Character vector of covariates.}
\item{anc.covars}{NULL or a character vector of ancillary covariates used to impute missing
                  values of \code{covars}
                  for subjects not in the nested case-control sample. 
                  If NULL, then random noise will be used for the imputation (see details).}
\item{risk.time}{Projection time for pure risk.}
\item{risk.time0}{Initial projection time for pure risk. Only used \code{time.vars} contains
                  two time variables. The default is 0.}
\item{nCntlPerCase}{Number of controls per case. The default is 1.}
\item{inclProb.var}{NULL or a variable in \code{data} giving the probability of being included in 
                    the nested case-control sample. 
                    This can be computed using the function \code{multipleNCC::KMprob}.
                    If NULL, then it is internally computed based on \code{outcome.var},
                    \code{time.vars}, and \code{nCntlPerCase}. The default is NULL.}
\item{print}{0 or 1 to print information. The default is 1.}
\item{min.inclProb}{Positive value \code{<= 1} giving the minimum inclusion probability for 
                    being in the nested case-control sample. Any inclusion probability 
                    less than \code{min.inclProb} will be set to \code{min.inclProb}.
                    This cannot be 0, since weights will be formed by taking the reciprocals
                    of the inclusion probabilities.
                    The default is 1e-6.}
\item{control}{See \code{\link{control}}.}
}
\value{
  A list containing two fitted \code{\link[survival]{coxph}} models: 
       \code{with.calibration} and \code{without.calibration}. 
}
\description{
  This function fits the Cox proportional hazards model from a nested case-control study
  using weight calibration.
}
\details{
  \bold{NOTE:} If \code{anc.covars = NULL}, then a seed must be set in order to reproduce
              the results. See Step 1 below.\cr

  The algorithm is as follows: \cr

  \bold{Step 1:} Obtain predictions for missing data \cr
  For any variable in \code{covars} with missing data from phase 1,
    fit a weighted generalized linear model among the nested case-control subjects (phase 2)
    with that variable as the outcome adjusting for the 
    other covariates in \code{covars} and \code{anc.covars}.
    The weights in the model are the reciprocals of the inclusion probabilities (\code{inclProb.var}).
    If \code{anc.covars = NULL}, then \code{runif(nrow(data))} will be used as an ancillary 
    covariate. \cr

  \bold{Step 2:} Create auxiliary statistics (influence functions) \cr

  \bold{Step 3:} Calibrate the design weights \cr

  \bold{Step 4:} Fit the Cox model with and without the calibrated weights. \cr


}
\seealso{ \code{\link{addHazard}}, \code{\link{diagnosticTests}} }
\keyword{model, Cox}

\examples{

  data(sample_data, package="WeightCalibSurvival")

  # Set the input arguments
  ncc.subset       <- sample_data[, "ind.ph2"] %in% 1
  outcome.var      <- "ind.fail"
  time.vars        <- "eventime"
  covars           <- c("X1", "X2", "Z1", "Z2")
  anc.covars       <- "U"
  risk.time        <- 8
  inclProb.var     <- "incl.prob"

  coxHazard(sample_data, ncc.subset, outcome.var, time.vars, covars, 
            anc.covars, risk.time, inclProb.var=inclProb.var)

}

\author{
  Yei Eun Shin \email{syeeun@gmail.com}
}

\references{
  Shin YE, Pfeiffer RM, Graubard BI, Gail MH. Weight calibration to improve
  the efficiency of pure risk estimates from case-control
  samples nested in a cohort. Biometrics. 2019;1-11.
  https://doi.org/10.1111/biom.13209
}


