\name{diagnosticTests}
\alias{diagnosticTests}
\title{Diagnostic Tests for Additive Hazards and Cox Proportional Hazards}
\usage{
diagnosticTests(data, ncc.subset, outcome.var, time.vars, 
                timeDep.covars, timeIndep.covars, nCntlPerCase=1, 
                inclProb.var=NULL, start.time=0, print=TRUE, plot=TRUE, 
                min.inclProb=1e-6) 
}
\arguments{
\item{data}{Data frame containing all the data for analysis.}
\item{ncc.subset}{Logical vector giving the subjects in the nested case-control sample.
                  The length must be equal to the number of rows in \code{data}. }
\item{outcome.var}{Binary outcome variable in \code{data} (1 = case, 0 = non-case).}
\item{time.vars}{One or two time-to-event variables in \code{data}.}
\item{timeDep.covars}{Character vector of covariates with time-varying effects.}
\item{timeIndep.covars}{Character vector of covariates with time-invariant effects.}
\item{nCntlPerCase}{Number of controls per case. 
                    This is only used if \code{inclProb.var = NULL}. The default is 1.}
\item{inclProb.var}{NULL or a variable in \code{data} giving the probability of being included in 
                    the nested case-control sample. 
                    This can be computed using the function \code{multipleNCC::KMprob}.
                    If NULL, then it is internally computed based on \code{outcome.var},
                    \code{time.vars}, and \code{nCntlPerCase}. The default is NULL.}
\item{start.time}{The starting time used to test for constant effects. The default is 0.}
\item{print}{TRUE or FALSE to print information. The default is TRUE.}
\item{plot}{TRUE or FALSE to create plots. The default is TRUE.}
\item{min.inclProb}{Positive value \code{<= 1} giving the minimum inclusion probability for 
                    being in the nested case-control sample. Any inclusion probability 
                    less than \code{min.inclProb} will be set to \code{min.inclProb}.
                    This cannot be 0, since weights will be formed by taking the reciprocals
                    of the inclusion probabilities.
                    The default is 1e-6.}
}
\value{
  A list containing two tables: \code{propHaz.test} and \code{addHaz.test}.
}
\description{
  Perform diagnostic tests for an additive hazard model and the proportional
  hazards assumption. 
}
\details{
  To test for proportional hazards, the \code{\link[survival]{cox.zph}}
  function is used. To determine if an additive hazards model is appropriate,
  the \code{\link[timereg]{aalen}} function is used to perform a
  Kolmogorov-Smirnov test for time-varying effects.
}
\seealso{ \code{\link{addHazard}}, \code{\link{coxHazard}} }
\keyword{test}

\examples{

  data(sample_data, package="WeightCalibSurvival")

  # Set the input arguments
  ncc.subset       <- sample_data[, "ind.ph2"] %in% 1
  outcome.var      <- "ind.fail"
  time.vars        <- "eventime"
  timeDep.covars   <- c("X1", "X2")
  timeIndep.covars <- c("Z1", "Z2")
  inclProb.var     <- "incl.prob"

  ret <- diagnosticTests(sample_data, ncc.subset, outcome.var, time.vars, 
                  timeDep.covars, timeIndep.covars, inclProb.var=inclProb.var)
  print(ret)
}

\author{
  Yei Eun Shin \email{syeeun@gmail.com}
}

\references{
  Shin YE, Pfeiffer RM Graubard BI, Gail MH. Weight calibration to
  improve efficiency for estimating pure risks from
  the additive hazards model with the nested
  case-control design. Biometrics. 2020;1-13.
  https://doi.org/10.1111/biom.13413
}


